@@ -13,11 +13,11 @@ Rex::Virtualization::CBSD - CBSD virtualization module for bhyve
 
 =head1 VERSION
 
-Version 0.0.1
+Version 1.0.0
 
 =cut
 
-our $VERSION = '0.0.1';
+our $VERSION = '1.0.0';
 
 =head1 SYNOPSIS
 
@@ -43,7 +43,7 @@ our $VERSION = '0.0.1';
     foreach my $vm_info_key (@{keys(%vm_info)}){
         print $vm_info_key.": ".$vm_info{$vm_info_key}."\n";
     }
-
+    
     # stop the VM foo
     vm 'stop' => 'foo';
     
@@ -68,7 +68,9 @@ sub new {
 
 =head1 Methods
 
-=head2 cbsd_base_dir
+=head2 General
+
+=head3 cbsd_base_dir
 
 This returns the CBSD base dir that the install is stored in.
 
@@ -78,7 +80,77 @@ This will die upon error.
 
     my $cbsd_base_dir=vm 'cbsd_base_dir'
 
-=head2 create
+=head3 freejname
+
+Gets the next available VM name.
+
+One argument is required and that is the base VM to use.
+
+The optional argument 'lease_time' may be used to specify the number
+of seconds a lease for the VM name should last. The default is 30.
+
+    vm 'freejname' => 'foo';
+    
+    # the same thing, but with a 60 second lease time
+    vm 'freejname' => 'foo', lease_time => '60';
+
+=head2 bhyve
+
+=head3 bcheckpoint_create
+
+Creates a checkpoint via the command below.
+
+    cbsd bcheckpoint mode=create jname=$vm suspend=$suspend name=$name
+
+The only required option is the key vm.
+
+    vm - VM to checkpoint
+    
+    name - name of checkpoint. by default: 'checkpoint'.
+    
+    suspend=  - when set to 1 then turn off the domain immediately after checkpoint,
+                for disk consistency. By default - 0, create checkpoint only.
+
+This will die upon error.
+
+    vm 'bcheckpoint_create', vm=>'foo';
+
+=head3 bcheckpoint_destroyall
+
+Removes all checkpoints via the command below.
+
+    cbsd bcheckpoint mode=destroyall jname=$vm
+
+One argument is taken and that is the name of the VM.
+
+This will die upon error.
+
+    # removes all checkpoints for the VM 'foo'
+    vm 'bcheckpoint_destroyall', 'foo';
+
+=head3 bclone
+
+This closnes a VM.
+
+    old - The VM to clone.
+    
+    new - The name of the new VM.
+    
+    checkstate - 0 do not check for VM online. Default is 1 - check
+    
+    promote - Promotes clone to no longer be dependent from origin: 0 or 1.
+              Default is 0 (not promote). This means new new VM operates in
+              copy-on-write mode and will be promoted upon removal of the
+              original.
+    
+    mac_reinit - 0,1 (default 1). 0 - leave old MAC.
+                 1 set mac to re-generate
+
+This will will die upon error.
+
+    vm 'bclone' old => 'foo', 'new' => 'bar';
+
+=head3 bcreate
 
 Creates a new VM.
 
@@ -154,7 +226,7 @@ This will die upon a error.
                               'vm_cpus'=>'1',
                               'imgsize'=>'10g';
 
-=head2 disk_list
+=head3 bdisk_list
 
 This returns a list of disks setup for use with Bhyve in CBSD via parsing
 the output of the command below.
@@ -183,7 +255,7 @@ This dies upon failure.
 
     my @disks
     eval{
-        @disks=vm 'disk_list';
+        @disks=vm 'bdisk_list';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to the disk list... '.$error);
@@ -191,21 +263,7 @@ This dies upon failure.
     
     print Dumper(\@disks);
 
-=head2 freejname
-
-Gets the next available VM name.
-
-One argument is required and that is the base VM to use.
-
-The optional argument 'lease_time' may be used to specify the number
-of seconds a lease for the VM name should last. The default is 30.
-
-    vm 'freejname' => 'foo';
-    
-    # the same thing, but with a 60 second lease time
-    vm 'freejname' => 'foo', lease_time => '60';
-
-=head2 info
+=head3 binfo
 
 This fetches the available configuration information for a VM via
 the command below.
@@ -216,7 +274,7 @@ The returned value is a flat hash of key value pairs.
 
     my %vm_info
     eval{
-        %vm_info=vm 'info' => 'foo';
+        %vm_info=vm 'binfo' => 'foo';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to get settings for the VM... '.$error);
@@ -226,10 +284,14 @@ The returned value is a flat hash of key value pairs.
         print $vm_info_key.": ".$vm_info{$vm_info_key}."\n";
     }
 
-=head2 list
+=head3 blist
 
 List available VMs.
 
+The command used is...
+
+    cbsd bls display=nodename,jname,jid,vm_ram,vm_curmem,vm_cpus,pcpu,vm_os_type,ip4_addr,status,vnc,path header=0
+
 The returned array is a hash of hashes. The first level hash is the jname.
 
     nodename - The node name that this is set to run on.
@@ -260,7 +322,7 @@ This dies upon failure.
 
     my %vm_list;
     eval{
-        %vm_list=vm 'list';
+        %vm_list=vm 'blist';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to list the VM... '.$error);
@@ -284,10 +346,14 @@ This dies upon failure.
             "\n"
     }
 
-=head2 nic_list
+=head3 bnic_list
 
 List configured NICs.
 
+The command used is as below...
+
+    cbsd bhyve-nic-list display=nodename,jname,nic_driver,nic_parent,nic_hwaddr,nic_address,nic_mtu,nic_persistent,nic_ratelimit header=0
+
 This returned data is a array of hashes.
 
 The keys are as below.
@@ -315,7 +381,7 @@ This dies upon failure.
 
     my @nics
     eval{
-        @nics=vm nic_list;
+        @nics=vm 'bnic_list';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to the NIC list... '.$error);
@@ -323,7 +389,7 @@ This dies upon failure.
     
     print Dumper(\@nics);
 
-=head2 pause
+=head3 bpause
 
 This pauses a VM in question. The following modes are available. If no
 more is specified, audo is used.
@@ -334,21 +400,25 @@ more is specified, audo is used.
 
 The command called is as below.
 
-    cbsd bpause $vm mode=$mode
+    cbsd 'bpause' => $vm mode=>$mode
 
 This dies upon failure.
 
     eval{
-        vm 'pause' => 'foo';
+        vm 'bpause' => 'foo';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to pause the VM foo... '.$error);
     }
 
-=head2 pci_list
+=head3 bpci_list
 
 List configured PCI devices for a VM.
 
+The command used is as below.
+
+    cbsd bpcibus mode=list jname=$vm
+
 This returned data is a array of hashes.
 
 The keys are as below.
@@ -369,7 +439,7 @@ This dies upon failure.
 
     my @devices
     eval{
-        @devices=vm 'nic_list' => 'foo';
+        @devices=vm 'bnic_list' => 'foo';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to the PCI device list... '.$error);
@@ -377,24 +447,24 @@ This dies upon failure.
     
     print Dumper(\@devices);
 
-=head2 remove
+=head3 bremove
 
 This removes the selected VM and remove the data. This is done via the command...
 
-    cbsd bdestroy $vm
+    cbsd bremove $vm
 
 One argument is taken and that is the name of the VM.
 
 This dies upon failure.
 
     eval{
-        vm 'remove' => 'foo'
+        vm 'bremove' => 'foo'
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to remove the VM foo... '.$error);
     }
 
-=head2 restart
+=head3 brestart
 
 This restarts the selected VM. This is done via the command...
 
@@ -405,13 +475,13 @@ One argument is taken and that is the name of the VM.
 This dies upon failure.
 
     eval{
-        vm 'restart' => 'foo'
+        vm 'brestart' => 'foo'
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to restart the VM foo... '.$error);
     }
 
-=head2 set
+=head3 bset
 
 This sets various settings for a VM via the use of...
 
@@ -424,30 +494,122 @@ not consider non-existent variables such as 'foofoo' to be a failure
 and silently ignores those.
 
     # set the the VM foo to boot from net with a resolution of 800x600
-    vm 'set' => 'foo',
+    vm 'bset' => 'foo',
         vm_boot => 'net',
         bhyve_vnc_resolution => '800x600';
 
-=head2 start
+=head3 bsnapshot_create
+
+This creates a disk snapshot of the specified VM via the command below.
+
+    cbsd bsnapshot mode=create jname=$vm  snapname=$name
+
+The following argument taken.
+
+    vm - The VM to snapshot. This is required.
+
+    name - The snapshot name. If not specified it defaults to snapshot.
+
+This will die upon error.
+
+    vm 'bsnapshot_create', vm=>'foo', name=>'aSnapshot';
+
+=head3 bsnapshot_list
+
+This lists all the snapshots via the command below.
+
+    cbsd bsnapshot mode=list display=jname,snapname,creation,refer header=0
+
+No arguments are taken.
+
+The return is a array of hashes. The hash keys are as below.
+
+    vm - Name of the VM the snapshot is for.
+    
+    name - Name of the snapshot.
+    
+    creation - The creation date. The format is YYYY-MM-DD__hh:mm.
+    
+    refer - ?
+
+This will die upon error.
+
+    my @snaps=vm 'bsnapshot_list';
+    print Dumper \@snaps;
+
+=head3 bsnapshot_remove
+
+This removes the specified snapshot for the specified VM.
+
+    cbsd bsnapshot mode=destroy jname=$vm  snapname=$name
+
+The following argument required.
+
+    vm - The VM to snapshot. This is required.
+
+    name - The snapshot name.
+
+This will die upon error.
+
+    vm 'bsnapshot_remove', vm=>'foo', name=>'aSnapshot';
+
+=head3 bsnapshot_removeall
+
+This removes specified snapshot for the specified VM.
+
+    cbsd bsnapshot mode=destroyall jname=$vm
+
+The following argument required.
+
+    vm - The VM to remove all snapshots for.
+
+This will die upon error.
+
+    vm 'bsnapshot_removeall', vm=>'foo';
+
+=head3 bsnapshot_rollback
+
+This rolls the disks for a VM back to a specified snapshot.
+
+    cbsd bsnapshot mode=rollback jname=$vm  snapname=$name
+
+The following argument required.
+
+    vm - The VM to rollback.
+
+    name - The snapshot name.
+
+This will die upon error.
+
+    vm 'bsnapshot_rollback', vm=>'foo', name=>'aSnapshot';
+
+=head3 bstart
 
 This starts a VM. This is done via the command...
 
     cbsd bstart jname=$vm
 
-One argument is taken and that is the name of the VM. If '*' or 'vm*' then
+One argument is required and that is the name of the VM. If '*' or 'vm*' then
 start all VM whose names begin with 'vm', e.g. 'vm1', 'vm2'...
 
+The following options may be used.
+
+    checkpoint - The name of the checkpoint to start the VM using.
+
 This dies upon failure.
 
     eval{
-        vm 'start' => 'foo'
+        vm 'bstart' => 'foo';
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to start the VM foo... '.$error);
     }
 
+    # starts foo from the checkpoint named checkpoint
+    vm 'bstart' => 'foo', checkpoint=>'checkpoint';
 
-=head2 stop
+
+=head3 bstop
 
 This stops a VM. This is done via the command below...
 
@@ -465,14 +627,69 @@ The following options are optional.
 This dies upon failure.
 
     eval{
-        vm 'stop' => 'foo',
+        vm 'bstop' => 'foo',
             hard_timeout => 60;
     } or do {
         my $error = $@ || 'Unknown failure';
         warn('Failed to stop the VM foo... '.$error);
     }
 
-=head2 vm_os_profiles
+=head3 p9shares_add
+
+This adds a p9 share for the specified VM. This is done via
+
+        cbsd bhyve-p9shares mode=attach jname=$vm p9device=$device p9path=$path
+
+The keys below are required.
+
+    vm - Name of the VM the share is for.
+    
+    device - p9 device name, one word.
+    
+    path - The shared path.
+
+This will die upon error.
+
+    vm 'p9shares_add', vm=>'foo', device=>'arc', path='/arc';
+
+=head3 p9shares_list
+
+This lists the configured p9 shares. This is fetched using the command below.
+
+    cbsd bhyve-p9shares mode=list header=0 display=jname,p9device,p9path
+
+No arguments are taken.
+
+The returned data is a array of hashes. The hash keys are as below.
+
+    vm - Name of the VM the share is for.
+    
+    device - p9 device name, one word.
+    
+    path - The shared path.
+
+This will die upon error.
+
+    my @shares=vm 'p9shares_list';
+    print Dumper \@shares;
+
+=head3 p9shares_rm
+
+This removes a p9 share for the specified VM. This is done via
+
+        cbsd bhyve-p9shares mode=deattach jname=$vm p9device=$device
+
+The keys below are required.
+
+    vm - Name of the VM the share is for.
+    
+    device - p9 device name, one word.
+
+This will die upon error.
+
+    vm 'p9shares_add', vm=>'foo', device=>'arc';
+
+=head3 vm_os_profiles
 
 Get the VM OS profiles for a specified OS type.
 
@@ -494,7 +711,7 @@ This will die upon failure.
     @profiles=vm 'vm_os_profiles' => 'freebsd', cloudinit=>1;
     print Dumper @profiles;
 
-=head2 vm_os_profiles_hash
+=head3 vm_os_profiles_hash
 
 Get the VM OS profiles for a specified OS type.
 
@@ -517,7 +734,7 @@ This will die upon failure.
     my %os_profiles=vm 'vm_os_profiles_hash', cloudinit=>1;
     print Dumper %os_profiles;
 
-=head2 vm_os_types
+=head3 vm_os_types
 
 Get the VM OS types there are profiles for.
 
